#!/usr/bin/perl -w -I(prefix)/raqdevil/perl -I(prefix)/raqdevil/handlers/base/network
#
# $Id: rewrite-ifcfg.pl,v 1.9 2005/12/13 20:09:30 dodell Exp $
#
# (c) 2000, 2001 Sun Microsystems, Inc.
# (c) 2005 Doug White
# (c) 2005 Devon H. O'Dell, iXsystems, All rights reserved.
#

use CCE;
use Sauce::Util;
use Network;

my $DEBUG = 0;

my $cce = new CCE;
$cce->connectfd();

my $oid = $cce->event_oid();
my $obj = $cce->event_object();
my $new = $cce->event_new();
my $old_net = $cce->event_old();

my $device = $obj->{'device'};
my $parent = Network::parentof($device);
my $is_alias = $device =~ /:/;
my $ipaddr = $obj->{'enabled'} ? $obj->{'ipaddr'} : '0.0.0.0';
my $netmask = $obj->{'netmask'};
my $enabled = $obj->{'enabled'} && $ipaddr;
my $onboot = $enabled ? 'yes' : 'no';

# this checks ipaddr here, because ipaddr is not optional
# and there is never a need to write an ifcfg file until
# the ipaddr field for a Network object is filled in
# this handler doesn't run on Network._DESTROY
if (not $obj->{ipaddr})
{
	$cce->bye('SUCCESS');
	exit(0);
}

if ($is_alias) {
	my @addrs;
	my @netmasks;
	my $i = 0;
	my $innet = 0;

	open (IFCFGP, "/sbin/ifconfig $parent |");
	while (<IFCFGP>) {
		if (/\sinet[^6]/) {
			($addrs[$i], $netmasks[$i]) = /inet\s(\d+\.\d+\.\d+\.\d+)\snetmask\s0x(\S{6}[0-9a-f][0-9a-e])/;
			my $haddr = $netmasks[$i];
			$netmasks[$i] = sprintf("%d.%d.%d.%d",
		    	hex(substr($haddr, 0, 2)),
		    	hex(substr($haddr, 2, 2)),
		    	hex(substr($haddr, 4, 2)),
		    	hex(substr($haddr, 6, 2)));
			$i++;
		}
	}
	close (IFCFGP);

	while ($i--) {
		if (netinnet($addrs[$i], $netmasks[$i], $ipaddr, $netmask)) {
			$innet = 1;
			last;
		}
	}

	if ($innet) {
		$cmd = "ifconfig $parent alias $ipaddr netmask 255.255.255.255";
	} else {
		$cmd = "ifconfig $parent alias $ipaddr netmask $netmask";
	}
} else {
	$cmd = "ifconfig $parent inet $ipaddr netmask $netmask";
}

$cmd .= "\n";

# We can get away with this because ifconfig IF delete will
# delete the last entry. This way, we delete all of the IP
# information from the real interface before editing the
# start_if file. This ensures that we will only have the
# correct IP information left over when the start_if file
# is executed thereafter.
open (STARTIF, "< /etc/start_if.$parent");
system("ifconfig $parent delete") while (<STARTIF>);

# Drop the new config in the new start_if file
my $ok = Sauce::Util::editfile("/etc/start_if.$parent", *edit_startif, $cmd, $old_net->{ipaddr}, $enabled);

system("/bin/sh /etc/start_if.$parent");

$cce->bye('SUCCESS');
exit(0);

# This subroutine calculates whether one network is to be found within another.
# In FreeBSD, we require IP address aliases to have a netmask with all bits
# set if it is on the same network as another address on the interface.
# (Indeed, one should not have IPs on the same network on multiple interfaces,
# but that is to be expected).
sub netinnet
{
	my ($mainIP, $mainNM, $aliasIP, $aliasNM) = @_;

	my ($bMainIP, $bMainMask) = binnet($mainIP, $mainNM);
	my ($bAliasIP, $bAliasMask) = binnet($aliasIP, $aliasNM);

	my $mainNet = $bMainIP & $bMainMask;
	my $aliasNet = $bAliasIP & $bAliasMask;

	if ($mainNet eq $aliasNet) {
		return 1;
	}

	return 0;
}

# Convert a dotted quad number to its binary counterpart
sub binnet
{
	my ($q1, $q2) = @_;

	my $bin1 = pack('CCCC', split(/\./, $q1));
	my $bin2 = pack('CCCC', split(/\./, $q2));

	return ($bin1, $bin2);
}

sub edit_startif {
	my($in, $out, $cmd, $oldip, $enabled) = @_;
	my $notOut = 1;

	while(<$in>) {
		if(/\s$oldip\s/) {
			if($enabled) {
				$notOut = 0;
				print $out $cmd;
			}
		} else {
			print $out $_;
		}
	}

	if ($notOut && $enabled) {
		print $out $cmd;
	}

	return 1;
}
